import { PropertyId } from "../../properties/property-id";
import { PropertyScalar } from "../../properties/property-scalar";
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { AbstractProperty } from "../../properties/abstract-property";
import { ValueConverter } from "../../value-converter/value-converter";
import { Choice } from "../../properties/choice";
import { ValueProvider } from "../../provider/value-provider/value-provider";
import { PropertyScalarRuleBinding } from "./property-scalar-rule-binding";
import { PropertyTemplate } from "../../properties/factory/property-template";
import { EmptyValueFcn } from "../../provider/value-provider/empty-value-fcn";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyScalarWithChoices } from "../../properties/property-scalar-with-choices";
import { ListIndex } from "../../properties/factory/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
import { Rule } from "../../rules/rule";
export interface PropertyScalarValueConfig<T> {
    valueConverter?: ValueConverter<T>;
    initialValue?: T | null;
    emptyValueFcn?: EmptyValueFcn<T>;
}
export interface PropertyScalarConfig {
    placeholder?: string;
    label?: string;
    labelAndPlaceholder?: string;
}
export declare class PropertyScalarBuilder {
    private readonly propertyScalar;
    private readonly bindPropertScalar;
    private readonly defaultEmptyChoice;
    constructor(propertyScalar: <T>(id: PropertyId, provider: ValueProvider<T>, emptyValueFcn: EmptyValueFcn<T>, converter: ValueConverter<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: PropertyScalarConfig & PropertyScalarValueConfig<T> & {
        backpressure?: BackpressureConfig;
    }, ownedProperties?: readonly AbstractProperty[]) => PropertyScalarImpl<T>, bindPropertScalar: <T>(prop: PropertyScalar<T>) => PropertyScalarRuleBinding<T>, defaultEmptyChoice: Choice<any> | undefined);
    template<D>(id: string, factory: (scalarBuilder: PropertyScalarBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<PropertyScalar<D>>) => PropertyScalar<D>): PropertyTemplate<PropertyScalar<D>, D>;
    isLike<T>(template: PropertyScalar<T>, valueProvider?: ValueProvider<T>): PropertyScalar<T>;
    simpleProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, config?: PropertyScalarConfig & {
        emptyValueFcn?: EmptyValueFcn<T>;
        initialValue?: T | null;
    }): PropertyScalar<T>;
    stringProperty(id: PropertyId, config?: PropertyScalarConfig & PropertyScalarValueConfig<string>): PropertyScalar<string>;
    numberProperty(id: PropertyId, config?: PropertyScalarConfig & PropertyScalarValueConfig<number> & {
        zeroIsConsideredAsEmpty?: boolean;
    }): PropertyScalar<number>;
    booleanProperty(id: PropertyId, config?: PropertyScalarConfig & PropertyScalarValueConfig<boolean>): PropertyScalar<boolean>;
    dateProperty(id: PropertyId, config?: PropertyScalarConfig & PropertyScalarValueConfig<Date>): PropertyScalar<Date>;
    private defaultInitValue;
    select: {
        static: <T>(id: PropertyId, choices: Choice<T>[], config?: (PropertyScalarConfig & {
            emptyChoice?: Choice<T> | undefined;
        }) | undefined) => PropertyScalarWithChoices<T, T>;
        derived: <T_1, Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => (config: PropertyScalarConfig & {
            derive: (...dependencies: Dependencies) => Choice<T_1>[];
            emptyChoice?: Choice<T_1> | undefined;
        }) => PropertyScalarWithChoices<T_1, T_1>;
        asyncDerived: <T_2, Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => (config: PropertyScalarConfig & {
            deriveAsync: (...dependencies: Dependencies_1) => Promise<Choice<T_2>[]>;
            emptyChoice?: Choice<T_2> | undefined;
            backpressure?: BackpressureConfig | undefined;
        }) => PropertyScalarWithChoices<T_2, T_2>;
        withPropertySource: <T_3>(id: PropertyId, choicesSource: PropertyScalar<Choice<T_3>[]>, config?: (PropertyScalarConfig & {
            emptyChoice?: Choice<T_3> | undefined;
        }) | undefined) => PropertyScalarWithChoices<T_3, T_3>;
    };
    typeahead: {
        async: <T>(id: PropertyId, config: PropertyScalarConfig & {
            fetchChoices: (currentText: string) => Promise<Choice<T>[]>;
            minimumTextLength?: number | undefined;
        }) => PropertyScalarWithChoices<[T | null, string], T>;
        withPropertySource: <T_1>(id: PropertyId, inputSource: PropertyScalar<string>, choicesSource: PropertyScalar<Choice<T_1>[]>, config?: PropertyScalarConfig | undefined) => PropertyScalarWithChoices<[T_1 | null, string], T_1>;
    };
    objectProperty<T, O>(id: PropertyId, valueConverter: ValueConverter<T>, obj: O, config: PropertyScalarConfig & {
        get: (obj: O) => T | null;
        set: (obj: O, val: T | null) => void;
        emptyValueFcn?: EmptyValueFcn<T>;
    }): PropertyScalar<T>;
    derived: {
        sync: <T, Dependencies extends readonly AbstractProperty[]>(id: PropertyId, valueConverter: ValueConverter<T>, ...dependencies: Dependencies) => (config: PropertyScalarConfig & {
            derive: Rule<[...dependencies: Dependencies], T | null>;
            inverse?: ((val: T | null, ...dependencies: Dependencies) => void) | undefined;
            emptyValueFcn?: EmptyValueFcn<T> | undefined;
        }) => PropertyScalar<T>;
        async: <T_1, Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, valueConverter: ValueConverter<T_1>, ...dependencies: Dependencies_1) => (config: PropertyScalarConfig & {
            deriveAsync: Rule<[...dependencies: Dependencies_1], Promise<T_1 | null>>;
            inverseAsync?: ((val: T_1 | null, ...dependencies: Dependencies_1) => Promise<void>) | undefined;
            backpressure?: BackpressureConfig | undefined;
            emptyValueFcn?: EmptyValueFcn<T_1> | undefined;
        }) => PropertyScalar<T_1>;
    };
    bind<T>(prop: PropertyScalar<T>): PropertyScalarRuleBinding<T>;
}
