"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyScalarBuilder = void 0;
const simple_value_provider_1 = require("../../provider/value-provider/simple-value-provider");
const select_value_converter_1 = require("../../value-converter/choices/select-value-converter");
const choice_value_provider_1 = require("../../provider/value-provider/choices/choice-value-provider");
const object_value_provider_1 = require("../../provider/value-provider/object-value-provider");
const common_value_converters_1 = require("../../value-converter/common-value-converters");
const derived_value_provider_1 = require("../../provider/value-provider/derived-value-provider");
const derived_async_value_provider_1 = require("../../provider/value-provider/derived-async-value-provider");
const empty_value_fcn_1 = require("../../provider/value-provider/empty-value-fcn");
const choice_list_converter_1 = require("../../value-converter/choices/choice-list-converter");
const assertions_1 = require("../../util/assertions/assertions");
const select_value_provider_1 = require("../../provider/value-provider/choices/select-value-provider");
const typeahead_value_provider_1 = require("../../provider/value-provider/choices/typeahead-value-provider");
const typeahead_value_converter_1 = require("../../value-converter/choices/typeahead-value-converter");
const property_scalar_with_choices_1 = require("../../properties/property-scalar-with-choices");
class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new choice_value_provider_1.ChoiceValueProvider(choices, emptyChoice);
                const converter = new select_value_converter_1.SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? empty_value_fcn_1.EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, undefined, config);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choices[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialValue();
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.sync(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            asyncDerived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            withPropertySource: (id, choicesSource, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new select_value_provider_1.SelectValueProvider(choicesSource, emptyChoice);
                const converter = new select_value_converter_1.SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? empty_value_fcn_1.EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], config, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choicesSource.getNonNullValue()[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialValue();
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, config) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), inputSourceProperty)({
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (config.minimumTextLength && text.length < config.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return config.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty, config);
            },
            withPropertySource: (id, inputSource, choicesSource, config) => {
                const provider = new typeahead_value_provider_1.TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new typeahead_value_converter_1.TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], config, [inputSource, choicesSource]);
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new derived_value_provider_1.DerivedValueProvider(dependencies, (deps) => config.derive(...deps), config === null || config === void 0 ? void 0 : config.inverse);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            },
            async: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new derived_async_value_provider_1.DerivedAsyncValueProvider(dependencies, (deps) => config.deriveAsync(...deps), config === null || config === void 0 ? void 0 : config.inverseAsync);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            }
        };
        assertions_1.assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        throw new Error('Not supported, yet');
    }
    simpleProperty(id, valueConverter, config) {
        var _a;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    stringProperty(id, config) {
        var _a, _b;
        config = this.defaultInitValue('', config);
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : common_value_converters_1.C.string.identity, undefined, config);
    }
    numberProperty(id, config) {
        var _a, _b;
        assertions_1.assertThat(() => !(config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) || !(config === null || config === void 0 ? void 0 : config.emptyValueFcn), () => `${id}: Provide either zeroIsConsideredAsEmpty or emptyValueFcn.`);
        config = this.defaultInitValue((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? 0 : null, config);
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : ((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn : empty_value_fcn_1.EmptyValueFcns.numberEmptyValueFcn), (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : common_value_converters_1.C.number.default, undefined, config);
    }
    booleanProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.booleanEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : common_value_converters_1.C.boolean.default, undefined, config);
    }
    dateProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : common_value_converters_1.C.date.iso, undefined, config);
    }
    defaultInitValue(initialValue, config) {
        if (!config) {
            return {
                initialValue
            };
        }
        if (config.initialValue === undefined) {
            config.initialValue = initialValue;
        }
        return config;
    }
    objectProperty(id, valueConverter, obj, config) {
        var _a;
        return this.propertyScalar(id, new object_value_provider_1.ObjectValueProvider(obj, config.get, config.set), (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
}
exports.PropertyScalarBuilder = PropertyScalarBuilder;
