import { PropertyScalar } from "../../properties/property-scalar";
import { AbstractProperty } from "../../properties/abstract-property";
import { PropertyDependencyOptions } from "../../dependency-graph/property-dependency";
import { AttributeId } from "../../attributes/attribute-id";
import { Rule } from "../../rules/rule";
import { ScalarValidator } from "../../validators/scalar-validator";
import { ValidationMessage } from "../../validators/validation-message";
export declare enum TextInterpreter {
    Plain = 0,
    Markdown = 1,
    Html = 2
}
export declare class PropertyScalarRuleBinding<T> {
    private readonly notEmptyIfRequiredValidator;
    private readonly addDependencies;
    private readonly property;
    constructor(property: PropertyScalar<T>, notEmptyIfRequiredValidator: ScalarValidator<unknown>, addDependencies: (from: readonly AbstractProperty[], to: AbstractProperty, options: PropertyDependencyOptions) => void);
    addScalarValidator(validator: ScalarValidator<T>): PropertyScalarRuleBinding<T>;
    addAsyncScalarValidator(validator: (property: PropertyScalar<T>) => Promise<ValidationMessage[] | undefined>): PropertyScalarRuleBinding<T>;
    defineInitialValue(value: T): PropertyScalarRuleBinding<T>;
    set<A>(attribudeId: AttributeId<A>, value: A): PropertyScalarRuleBinding<T>;
    define<A, Dependencies extends readonly AbstractProperty[]>(attribudeId: AttributeId<A>, ...dependencies: Dependencies): (value: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], A>) => PropertyScalarRuleBinding<T>;
    private defineAttributeFunction;
    setVisibility(isVisible: boolean): PropertyScalarRuleBinding<T>;
    defineVisibility<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (isVisible: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], boolean>) => PropertyScalarRuleBinding<T>;
    private defineVisibilityFunction;
    setRequiredIfVisible(required: boolean): PropertyScalarRuleBinding<T>;
    defineRequiredIfVisible<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (required: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], boolean>) => PropertyScalarRuleBinding<T>;
    private defineRequiredIfVisibleFunction;
    defineLabelAndPlaceholder(text: string): PropertyScalarRuleBinding<T>;
    defineLabel(text: string, textInterpreter?: TextInterpreter): PropertyScalarRuleBinding<T>;
    definePlaceholder(placeholder: string): PropertyScalarRuleBinding<T>;
    defineInfoText(text: string, textInterpreter?: TextInterpreter): PropertyScalarRuleBinding<T>;
    private interpreteText;
    onUpdated(updated: (self: PropertyScalar<T>) => void): this;
    setToInitialValueOnOtherPropertyChanged(...otherProperties: AbstractProperty[]): this;
}
