"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyScalarImpl = void 0;
const logger_1 = require("../util/logger/logger");
const abstract_property_impl_1 = require("./abstract-property-impl");
const assertions_1 = require("../util/assertions/assertions");
class PropertyScalarImpl extends abstract_property_impl_1.AbstractPropertyImpl {
    constructor(id, valueProvider, emptyValueFcn, valueConverter, updateHandler, backpressureConfig) {
        super(updateHandler, backpressureConfig);
        this.id = id;
        this.valueProvider = valueProvider;
        this.emptyValueFcn = emptyValueFcn;
        this.valueConverter = valueConverter;
        this.scalarValidators = [];
    }
    internallySyncUpdate() {
        if (this.valueProvider.shouldBeCached()) {
            this.currentValue = this.valueProvider.getValue();
        }
    }
    internallyAsyncUpdate() {
        const asyncPromise = this.valueProvider.getValue();
        assertions_1.assertThat(() => asyncPromise instanceof Promise, () => `${this.id}: Expect the value provider to return a promise since it is declared as asynchronous`);
        return {
            asyncPromise: asyncPromise,
            resolve: value => {
                this.currentValue = value;
            }
        };
    }
    getCurrentValue() {
        return this.currentValue !== undefined ? this.currentValue : this.valueProvider.getValue();
    }
    setToInitialValue() {
        const initialValue = this.initialValue !== undefined ? this.initialValue : null;
        if (this.valueProvider.shouldBeCached() || this.isAsynchronous()) {
            this.currentValue = initialValue;
        }
        if (!this.isReadOnly()) {
            this.valueProvider.setValue(initialValue);
        }
        logger_1.Logger.trace(() => `PropertyScalarImpl.setToInitialValue ${this.id}: ${initialValue}`);
    }
    getInitialValue() {
        var _a;
        return (_a = this.initialValue) !== null && _a !== void 0 ? _a : null;
    }
    getSpecialisedValidationResult() {
        return this.scalarValidators.reduce((res, sv) => {
            const msg = sv(this);
            if (msg) {
                res.push(msg);
            }
            return res;
        }, []);
    }
    addScalarValidator(validator) {
        this.scalarValidators.push(validator);
    }
    defineInitialValue(value) {
        this.initialValue = value;
    }
    defineLabel(label) {
        this.label = label;
    }
    definePlaceholder(placeholder) {
        this.placeholder = placeholder;
    }
    defineInfoText(infoText) {
        this.infoText = infoText;
    }
    defineVisibility(visibility) {
        this.visible = visibility;
    }
    defineRequiredIfVisible(mandatoriness) {
        this.required = mandatoriness;
    }
    defineAttribute(attribute) {
        if (!this.attributeMap) {
            this.attributeMap = new Map();
        }
        this.attributeMap.set(attribute.id, attribute);
    }
    getDisplayValue() {
        const current = this.getValue();
        return this.valueConverter.asDisplayValue(current);
    }
    setDisplayValue(value) {
        this.setValue(this.valueConverter.fromDisplayValue(value));
    }
    awaitDisplayValue() {
        return __awaiter(this, void 0, void 0, function* () {
            const current = yield this.awaitValue();
            return this.valueConverter.asDisplayValue(current);
        });
    }
    getNonNullValue() {
        var _a;
        return (_a = this.getValue()) !== null && _a !== void 0 ? _a : this.valueConverter.getNullFallbackValue();
    }
    getValue() {
        if (!this.isAsynchronous()) {
            this.checkAndTriggerUpdate();
        }
        return this.getCurrentValue();
    }
    setValue(value) {
        if (value !== this.getCurrentValue()) {
            this.valueProvider.setValue(value);
            this.needsAnUpdate();
        }
    }
    awaitValue() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.awaitAsyncUpdate();
            return this.getCurrentValue();
        });
    }
    get(id) {
        var _a, _b;
        return (_b = (_a = this.attributeMap) === null || _a === void 0 ? void 0 : _a.get(id)) === null || _b === void 0 ? void 0 : _b.getValue();
    }
    isRequired() {
        return !!this.required && this.required.getValue() && this.isVisible();
    }
    isVisible() {
        var _a, _b;
        return (_b = (_a = this.visible) === null || _a === void 0 ? void 0 : _a.getValue()) !== null && _b !== void 0 ? _b : true;
    }
    isEmpty() {
        return this.emptyValueFcn(this.getValue());
    }
    getLabel() {
        var _a;
        return (_a = this.label) !== null && _a !== void 0 ? _a : '';
    }
    getInfoText() {
        var _a;
        return (_a = this.infoText) !== null && _a !== void 0 ? _a : '';
    }
    getPlaceholder() {
        var _a;
        return (_a = this.placeholder) !== null && _a !== void 0 ? _a : '';
    }
    isAsynchronous() {
        return this.valueProvider.isAsynchronous();
    }
    isProcessing() {
        return this.valueProvider.isProcessing();
    }
    isReadOnly() {
        return this.valueProvider.isReadOnly();
    }
    exportData() {
        return this.getValue();
    }
    importData(data) {
        this.setValue(data);
    }
    compareData(a, b) {
        return a === b;
    }
}
exports.PropertyScalarImpl = PropertyScalarImpl;
