"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleListProvider = void 0;
const list_index_impl_1 = require("../../properties/factory/list-index-impl");
class SimpleListProvider {
    constructor(id, propertyTemplate, getSelected) {
        this.id = id;
        this.propertyTemplate = propertyTemplate;
        this.getSelected = getSelected;
        this.list = [];
        this.siblingCount = this.list.length;
        this.nxtId = 0;
    }
    getList() {
        return this.list.map(el => el.prop);
    }
    clearList() {
        this.list.splice(0, this.list.length);
    }
    addProperty(atIndex) {
        const propId = `${this.id}_${this.nxtId}`;
        this.nxtId++;
        const idx = atIndex !== null && atIndex !== void 0 ? atIndex : this.list.length;
        const index = new list_index_impl_1.ListIndexImpl(idx, this.list, this.getSelected);
        const prop = this.propertyTemplate(propId, index, this);
        if (atIndex !== undefined) {
            this.list.splice(atIndex, 0, { prop, index });
            this.adjustIndices(atIndex + 1);
        }
        else {
            this.list.push({ prop, index });
        }
        return prop;
    }
    getProperty(atIndex) {
        var _a;
        return (_a = this.list[atIndex]) === null || _a === void 0 ? void 0 : _a.prop;
    }
    removeByIndex(idx) {
        if (idx < 0 || idx >= this.list.length) {
            return;
        }
        else {
            const [removed] = this.list.splice(idx, 1);
            this.adjustIndices(idx);
            return removed === null || removed === void 0 ? void 0 : removed.prop;
        }
    }
    moveProperty(from, to) {
        if (from !== to) {
            const [prop] = this.list.splice(from, 1);
            if (prop) {
                this.list.splice(to, 0, prop);
                if (from < to) {
                    this.adjustIndices(from);
                }
                else {
                    this.adjustIndices(to);
                }
            }
        }
    }
    adjustIndices(start, end) {
        const to = end || this.list.length;
        for (let i = start !== null && start !== void 0 ? start : 0; i < to; i++) {
            this.list[i].index.index = i;
        }
    }
    isAsynchronous() {
        return false;
    }
    isProcessing() {
        return false;
    }
    shouldBeCached() {
        return false;
    }
    isReadOnly() {
        return false;
    }
    getSibling(atIndex) {
        return this.getProperty(atIndex);
    }
    someSibling(predicate) {
        return this.list.some((el) => predicate(el.prop, el.index.idx));
    }
    everySibling(predicate) {
        return this.list.every((el, index) => predicate(el.prop, index));
    }
    reduceSiblings(callbackfn, initialValue) {
        return this.list.reduce((res, el, i) => callbackfn(res, el.prop, i), initialValue);
    }
    filterSiblings(predicate) {
        return this.list.filter((el, i) => predicate(el.prop, i)).map(el => el.prop);
    }
}
exports.SimpleListProvider = SimpleListProvider;
