import { PropertyId } from "../properties/property-id";
import { AbstractProperty } from "../properties/abstract-property";
import { PropertyDependency, PropertyDependencyOptions } from "./property-dependency";
export declare type VisNodeGroup = 'scalar' | 'scalar-async' | 'group' | 'group-async' | 'list' | 'list-async';
export interface VisJsNode {
    id: number;
    label: string;
    group: VisNodeGroup;
}
export interface VisJsEdge {
    from: number;
    to: number;
    title?: string;
    dashes?: boolean | number[];
    color?: string;
    arrows?: {
        to?: {
            enabled: boolean;
            type: string;
        };
        from?: {
            enabled: boolean;
            type: string;
        };
    };
}
export interface OwnerRelation {
    addOwnerDependency(owner: AbstractProperty, ownedProperty: AbstractProperty, withValueDependency?: boolean): void;
}
export declare class DependencyGraph implements OwnerRelation {
    private readonly edgesMap;
    private readonly asyncEdgesMap;
    private readonly ownerMap;
    addDependency(from: AbstractProperty, to: AbstractProperty, options: PropertyDependencyOptions): void;
    addDependencies(from: readonly AbstractProperty[], to: AbstractProperty, options: PropertyDependencyOptions): void;
    addOwnerDependency(owner: AbstractProperty, ownedProperty: AbstractProperty, withValueDependency?: boolean): void;
    traverseDepthFirst(start: PropertyId, apply: (prop: AbstractProperty, dependency: PropertyDependency) => void, filter?: (dependency: PropertyDependency) => boolean, preventCycles?: boolean): void;
    getAsyncDependencies(propertyId: PropertyId): AbstractProperty[] | undefined;
    findCyclicDependencies(): PropertyId[] | null;
    private findCyclicDependenciesRecursive;
    createVisJsData(allProperties: AbstractProperty[]): {
        nodes: VisJsNode[];
        edges: VisJsEdge[];
        options: {
            nodes: {
                shape: string;
                size: number;
                font: {
                    size: number;
                };
                borderWidth: number;
                shadow: boolean;
            };
            groups: {
                scalar: {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                'scalar-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                group: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'group-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                list: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'list-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
            };
            edges: {
                width: number;
                shadow: boolean;
            };
        };
    };
    private createVisJsNodes;
    private propertyToVisNodeGroup;
    private createVisJsEdges;
}
