import { RuleEngine } from "../rule-engine";
import { AbstractProperty } from "../../properties/abstract-property";
import { DependencyGraph } from "../../dependency-graph/dependency-graph";
import { AbstractPropertyWithInternals } from "../../properties/abstract-property-impl";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { BuilderOptions } from "./builder-options";
import { AttributeId } from "../../attributes/attribute-id";
import { ListOfPropertiesBuilder } from "./list-of-properties-builder";
import { Validator } from "../../validators/validator";
export declare class Builder {
    private readonly ruleEngine;
    private readonly dependencyGraph;
    private readonly propertyMap;
    private get properties();
    private readonly notEmptyIfRequiredValidator;
    private readonly defaultEmptyChoice;
    private readonly defaultBackpressureConfig;
    readonly scalar: PropertyScalarBuilder;
    readonly group: GroupOfPropertiesBuilder;
    readonly trigger: TriggerBuilder;
    readonly list: ListOfPropertiesBuilder;
    constructor(options: BuilderOptions, ruleEngine: RuleEngine, dependencyGraph: DependencyGraph, propertyMap: {
        [id: string]: AbstractPropertyWithInternals<unknown>;
    });
    private propertyScalar;
    private bindPropertyScalar;
    private groupOfProperties;
    private listOfProperties;
    private addDependencies;
    private addProperty;
    defineCustomAttribute<A>(name: string): AttributeId<A>;
    bindValidator<Properties extends readonly AbstractProperty[]>(...properties: Properties): (validator: Validator<Properties>) => void;
    asVisJsData(): {
        nodes: import("../../dependency-graph/dependency-graph").VisJsNode[];
        edges: import("../../dependency-graph/dependency-graph").VisJsEdge[];
        options: {
            nodes: {
                shape: string;
                size: number;
                font: {
                    size: number;
                };
                borderWidth: number;
                shadow: boolean;
            };
            groups: {
                scalar: {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                'scalar-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                group: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'group-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                list: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'list-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
            };
            edges: {
                width: number;
                shadow: boolean;
            };
        };
    };
    generateNetworkGraphHtmlPage(): string;
    generateFactory(): string;
}
