import { AbstractProperty } from "./abstract-property";
import { ValueChangeListener } from "./value-change-listener";
import { Trigger } from "./trigger";
import { RuleEngineUpdateHandler } from "../engine/rule-engine-update-handler-impl";
import { ValidationMessage } from "../validators/validation-message";
import { PropertyDependency } from "../dependency-graph/property-dependency";
import { BackpressureConfig } from "./backpressure/backpressure-config";
import { ValidatorInstance } from "../engine/validation/validator-instance-impl";
import { AbstractDataProperty } from "./abstract-data-property";
export interface AbstractPropertyWithInternals<D> extends AbstractDataProperty<D> {
    internallyUpdate(): Promise<void>;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    internallyRequiresEagerUpdate(): boolean;
    addValidator<Properties extends readonly AbstractProperty[]>(validator: ValidatorInstance<Properties>): void;
}
export declare abstract class AbstractPropertyImpl<D> implements AbstractPropertyWithInternals<D> {
    protected updateHandler: RuleEngineUpdateHandler;
    private automaticallyUpdate;
    private manuallyTriggered?;
    private triggerListener?;
    private needsToRecompute?;
    private recomputingCount?;
    private currentRecomputing?;
    private updatedListeners?;
    private needsToRevalidate?;
    private validators?;
    private validationMessages;
    constructor(updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig);
    private readonly valueChangeListeners;
    abstract id: string;
    protected abstract internallySyncUpdate(): void;
    protected abstract internallyAsyncUpdate<V>(): {
        asyncPromise: Promise<V>;
        resolve: (value: V) => void;
    };
    protected awaitAsyncUpdate(): Promise<void>;
    private updatedPromise;
    private nextRecomputingCount;
    protected checkAndTriggerUpdate(): void;
    private getTriggerListener;
    internallyUpdate(): Promise<void>;
    private handleBackpressure;
    private handleAsyncUpdateWithDebounceTime;
    private handleAsyncUpdate;
    needsAnUpdate(notifyOthers?: boolean): void;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    internallyRequiresEagerUpdate(): boolean;
    protected abstract getSpecialisedValidationResult(): ValidationMessage[];
    addValidator<Properties extends readonly AbstractProperty[]>(validator: ValidatorInstance<Properties>): void;
    validate(): Promise<ValidationMessage[]>;
    isValid(): boolean;
    getValidationMessages(): ValidationMessage[];
    registerValueChangedListener(changed: ValueChangeListener): void;
    deregisterValueChangedListener(changed: ValueChangeListener): void;
    protected tellValueChangeListeners(fcn: (listener: ValueChangeListener, index: number) => void): void;
    setUpdateStrategy(strategy: 'Automatic' | 'Manually'): void;
    addUpdateTrigger(trigger: Trigger, setUpdateStrategyManually?: boolean): void;
    removeUpdateTrigger(trigger: Trigger): void;
    backpressureConfig?: BackpressureConfig;
    abstract isAsynchronous(): boolean;
    abstract isProcessing(): boolean;
    abstract isReadOnly(): boolean;
    abstract exportData(): D | null;
    abstract importData(data: D | null): void;
    abstract compareData(a: D | null, b: D | null): boolean;
}
