import { AbstractPropertyImpl } from "./abstract-property-impl";
export class ListOfPropertiesImpl extends AbstractPropertyImpl {
    constructor(id, listProvider, selectedIndices, isMultiSelect = false, updateHandler, ownerRelation) {
        super(updateHandler);
        this.id = id;
        this.listProvider = listProvider;
        this.selectedIndices = selectedIndices;
        this.isMultiSelect = isMultiSelect;
        this.ownerRelation = ownerRelation;
    }
    get length() {
        return this.listProvider.getList().length;
    }
    get list() {
        return this.listProvider.getList();
    }
    internallySyncUpdate() {
        throw new Error("Method not implemented.");
    }
    internallyAsyncUpdate() {
        throw new Error("Method not implemented.");
    }
    getSpecialisedValidationResult() {
        return [];
    }
    getProperty(index) {
        return this.listProvider.getProperty(index);
    }
    addProperty(property, atIndex, dontNotify) {
        const prop = this.listProvider.addProperty(atIndex);
        if (property) {
            prop.importData(property.exportData());
        }
        this.ownerRelation.addOwnerDependency(this, prop);
        if (!dontNotify) {
            this.needsAnUpdate();
        }
        return prop;
    }
    addProperties(properties, atIndex) {
        const res = [];
        const count = typeof properties === 'number' ? properties : properties.length;
        for (let c = 0; c < count; c++) {
            const property = typeof properties === 'number' ? undefined : properties[c];
            res.push(this.addProperty(property, atIndex, true));
        }
        this.needsAnUpdate();
        return res;
    }
    addPropertyData(data, atIndex) {
        const result = data.map((d, i) => {
            const index = atIndex !== undefined ? atIndex + i : undefined;
            const prop = this.listProvider.addProperty(index);
            prop.importData(d);
            this.ownerRelation.addOwnerDependency(this, prop);
            return prop;
        });
        this.needsAnUpdate();
        return result;
    }
    swapProperties(indexA, indexB) {
        if (indexA !== indexB) {
            this.listProvider.moveProperty(indexA, indexB);
            if (indexA < indexB) {
                this.listProvider.moveProperty(indexB - 1, indexA);
            }
            else {
                this.listProvider.moveProperty(indexB + 1, indexA);
            }
            const aIdx = this.selectedIndices.indexOf(indexA);
            const bIdx = this.selectedIndices.indexOf(indexB);
            if (aIdx >= 0) {
                if (bIdx === -1) {
                    this.selectedIndices.splice(aIdx, 1, indexB);
                }
            }
            else if (bIdx >= 0) {
                this.selectedIndices.splice(bIdx, 1, indexA);
            }
            this.needsAnUpdate();
        }
    }
    moveProperty(fromIndex, toIndex) {
        this.listProvider.moveProperty(fromIndex, toIndex);
        const selectedIdx = this.selectedIndices.indexOf(fromIndex);
        if (selectedIdx >= 0) {
            this.selectedIndices.splice(fromIndex, 1, toIndex);
        }
        this.needsAnUpdate();
    }
    removePropertyAtIndex(index) {
        const idx = this.selectedIndices.indexOf(index);
        if (idx >= 0) {
            this.selectedIndices.splice(idx, 1);
            for (let i = 0; i < this.selectedIndices.length; i++) {
                const selected = this.selectedIndices[i];
                if (selected > index) {
                    this.selectedIndices.splice(i, 1, selected - 1);
                }
            }
        }
        const result = this.listProvider.removeByIndex(index);
        if (result) {
            this.needsAnUpdate();
        }
        return result;
    }
    removeProperty(property) {
        const idx = this.listProvider.getList().findIndex(el => el.id === property.id);
        return idx >= 0 && this.removePropertyAtIndex(idx) !== undefined;
    }
    selectPropertyAtIndex(index) {
        if (this.isMultiSelect) {
            if (!this.selectedIndices.includes(index)) {
                this.selectedIndices.push(index);
                this.needsAnUpdate();
            }
        }
        else {
            this.selectedIndices.splice(0, this.selectProperty.length, index);
            this.needsAnUpdate();
        }
    }
    selectProperty(property) {
        const idx = this.listProvider.getList().findIndex(el => el.id === property.id);
        this.selectPropertyAtIndex(idx);
    }
    isPropertySelectedAtIndex(index) {
        return this.selectedIndices.includes(index);
    }
    isPropertySelected(property) {
        const idx = this.listProvider.getList().findIndex(el => el.id === property.id);
        return this.isPropertySelectedAtIndex(idx);
    }
    getSelectedIndices() {
        return [...this.selectedIndices];
    }
    getSelectedProperties() {
        return this.selectedIndices.map(index => ({ index, property: this.getProperty(index) }));
    }
    unselectPropertyAtIndex(index) {
        const idxOfIndex = this.selectedIndices.findIndex(selected => selected === index);
        if (idxOfIndex >= 0) {
            this.selectedIndices.splice(idxOfIndex, 1);
            this.needsAnUpdate();
        }
    }
    unselectProperty(property) {
        const idx = this.listProvider.getList().findIndex(el => el.id === property.id);
        this.unselectPropertyAtIndex(idx);
    }
    unselectAll() {
        this.selectedIndices.splice(0, this.selectedIndices.length);
        this.needsAnUpdate();
    }
    isValid() {
        return super.isValid() && true;
    }
    isProcessing() {
        return false;
    }
    isReadOnly() {
        return false;
    }
    isAsynchronous() {
        return false;
    }
    exportData() {
        return this.listProvider.getList().map(prop => prop.exportData());
    }
    importData(data) {
        this.listProvider.clearList();
        data.forEach((entry) => {
            const prop = this.listProvider.addProperty();
            prop.importData(entry);
        });
        this.needsAnUpdate();
    }
    compareData(a, b) {
        return true;
    }
}
