import { PropertyScalar } from "./property-scalar";
import { ValueProvider } from "../provider/value-provider/value-provider";
import { ValueConverter } from "../value-converter/value-converter";
import { AttributeId } from "../attributes/attribute-id";
import { Attribute } from "../attributes/attribute";
import { PropertyId } from "./property-id";
import { ScalarValidator } from "../validators/scalar-validator";
import { ValidationMessage } from "../validators/validation-message";
import { RuleEngineUpdateHandler } from "../engine/rule-engine-update-handler-impl";
import { AbstractPropertyImpl } from "./abstract-property-impl";
import { EmptyValueFcn } from "../provider/value-provider/empty-value-fcn";
import { BackpressureConfig } from "./backpressure/backpressure-config";
export declare class PropertyScalarImpl<T> extends AbstractPropertyImpl<T> implements PropertyScalar<T> {
    readonly id: PropertyId;
    private readonly valueProvider;
    private readonly emptyValueFcn;
    private readonly valueConverter;
    private currentValue?;
    protected initialValue?: T | null;
    private placeholder?;
    private infoText?;
    private label?;
    protected scalarValidators: ScalarValidator<T>[];
    protected attributeMap?: Map<AttributeId<unknown>, Attribute<any>>;
    protected visible?: Attribute<boolean>;
    protected required?: Attribute<boolean>;
    constructor(id: PropertyId, valueProvider: ValueProvider<T>, emptyValueFcn: EmptyValueFcn<T>, valueConverter: ValueConverter<T>, updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig);
    protected internallySyncUpdate(): void;
    protected internallyAsyncUpdate(): {
        asyncPromise: Promise<any>;
        resolve: (value: any) => void;
    };
    private getCurrentValue;
    setToInitialValue(): void;
    getInitialValue(): NonNullable<T> | null;
    protected getSpecialisedValidationResult(): ValidationMessage[];
    addScalarValidator(validator: ScalarValidator<T>): void;
    defineInitialValue(value: T | null): void;
    defineLabel(label: string): void;
    definePlaceholder(placeholder: string): void;
    defineInfoText(infoText: string): void;
    defineVisibility(visibility?: Attribute<boolean>): void;
    defineRequiredIfVisible(mandatoriness?: Attribute<boolean>): void;
    defineAttribute(attribute: Attribute<unknown>): void;
    getDisplayValue(): string;
    setDisplayValue(value: string | null): void;
    awaitDisplayValue(): Promise<string>;
    getNonNullValue(): T;
    getValue(): T | null;
    setValue(value: T | null): void;
    awaitValue(): Promise<T | null>;
    get<A>(id: AttributeId<A>): A | undefined;
    isRequired(): boolean;
    isVisible(): boolean;
    isEmpty(): boolean;
    getLabel(): string;
    getInfoText(): string;
    getPlaceholder(): string;
    isAsynchronous(): boolean;
    isProcessing(): boolean;
    isReadOnly(): boolean;
    exportData(): T | null;
    importData(data: T | null): void;
    compareData(a: T | null, b: T | null): boolean;
}
