export class LoggerInstance {
    constructor() {
        this.logLevel = LoggerInstance.DEBUG;
    }
    log(msg, level, optionalParams) {
        if (this.logLevel <= level) {
            if (msg instanceof Function) {
                msg = msg();
            }
            if (msg instanceof Object || msg instanceof Array) {
                msg = JSON.stringify(msg);
            }
            if (optionalParams === null || optionalParams === void 0 ? void 0 : optionalParams.length) {
                console.log(msg, ...optionalParams);
            }
            else {
                console.log(msg);
            }
        }
    }
    trace(msg) {
        this.log(msg, LoggerInstance.TRACE);
    }
    debug(msg) {
        this.log(msg, LoggerInstance.DEBUG);
    }
    info(msg) {
        this.log(msg, LoggerInstance.INFO);
    }
    warn(msg) {
        this.log(msg, LoggerInstance.WARN);
    }
    error(msg, error) {
        this.log(msg, LoggerInstance.ERROR, error);
    }
}
LoggerInstance.TRACE = 1;
LoggerInstance.DEBUG = 2;
LoggerInstance.INFO = 3;
LoggerInstance.WARN = 4;
LoggerInstance.ERROR = 5;
LoggerInstance.NONE = 6;
export const Logger = new LoggerInstance();
