import { ValueConverter } from "./value-converter";
export declare class StringValueConverter implements ValueConverter<string> {
    fromDisplayValue(value: string | null): string | null;
    asDisplayValue(value: string | null): string;
    getNullFallbackValue(): string;
}
export declare class UpperCaseStringValueConverter implements ValueConverter<string> {
    fromDisplayValue(value: string | null): string | null;
    asDisplayValue(value: string | null): string;
    getNullFallbackValue(): string;
}
export declare class NumberValueConverter implements ValueConverter<number> {
    fromDisplayValue(value: string | null): number | null;
    asDisplayValue(value: number | null): string;
    getNullFallbackValue(): number;
}
export declare class PrecisionNumberValueConverter implements ValueConverter<number> {
    protected precision: number;
    constructor(precision: number);
    fromDisplayValue(value: string | null): number | null;
    asDisplayValue(value: number | null): string;
    getNullFallbackValue(): number;
}
export declare class IntegerValueConverter implements ValueConverter<number> {
    fromDisplayValue(value: string | null): number | null;
    asDisplayValue(value: number | null): string;
    getNullFallbackValue(): number;
}
export declare class BooleanConverter implements ValueConverter<boolean> {
    private readonly trueString;
    private readonly falseString;
    private readonly loweredTrueString;
    constructor(trueString: string, falseString: string);
    fromDisplayValue(value: string | null): boolean | null;
    asDisplayValue(value: boolean | null): string;
    getNullFallbackValue(): boolean;
}
export declare class IsoDateConverter implements ValueConverter<Date> {
    fromDisplayValue(value: string | null): Date | null;
    asDisplayValue(value: Date | null): string;
    getNullFallbackValue(): Date;
}
export declare class GermanDateConverter implements ValueConverter<Date> {
    fromDisplayValue(value: string | null): Date | null;
    asDisplayValue(value: Date | null): string;
    getNullFallbackValue(): Date;
}
export declare const C: {
    string: {
        identity: StringValueConverter;
        upperCase: UpperCaseStringValueConverter;
    };
    number: {
        default: NumberValueConverter;
        withPrecision: (precision: number) => PrecisionNumberValueConverter;
        integer: IntegerValueConverter;
    };
    boolean: {
        default: BooleanConverter;
        jaNein: BooleanConverter;
    };
    date: {
        iso: IsoDateConverter;
        german: GermanDateConverter;
    };
};
